@@ -25,6 +25,7 @@ import (
 	"strings"
 	"time"
 
+	"github.com/bep/debounce"
 	"github.com/bep/logg"
 	"github.com/gohugoio/hugo/bufferpool"
 	"github.com/gohugoio/hugo/deps"
@@ -45,6 +46,7 @@ import (
 	"github.com/gohugoio/hugo/common/para"
 	"github.com/gohugoio/hugo/common/paths"
 	"github.com/gohugoio/hugo/common/rungroup"
+	"github.com/gohugoio/hugo/common/terminal"
 	"github.com/gohugoio/hugo/config"
 	"github.com/gohugoio/hugo/resources/page"
 	"github.com/gohugoio/hugo/resources/page/siteidentities"
@@ -58,9 +60,25 @@ import (
 // Build builds all sites. If filesystem events are provided,
 // this is considered to be a potential partial rebuild.
 func (h *HugoSites) Build(config BuildCfg, events ...fsnotify.Event) error {
+	if !h.isRebuild() && terminal.PrintANSIColors(os.Stdout) {
+		// Don't show progress for fast builds.
+		d := debounce.New(250 * time.Millisecond)
+		d(func() {
+			h.progressReporter.Start()
+			h.reportProgress(func() (state terminal.ProgressState, progress float64) {
+				// We don't know how many files to process below, so use the intermediate state as the first progress.
+				return terminal.ProgressIntermediate, 1.0
+			})
+		})
+		defer d(func() {})
+	}
+
 	infol := h.Log.InfoCommand("build")
 	defer loggers.TimeTrackf(infol, time.Now(), nil, "")
 	defer func() {
+		h.reportProgress(func() (state terminal.ProgressState, progress float64) {
+			return terminal.ProgressHidden, 1.0
+		})
 		h.buildCounter.Add(1)
 	}()
 
@@ -148,10 +166,15 @@ func (h *HugoSites) Build(config BuildCfg, events ...fsnotify.Event) error {
 			if err := h.process(ctx, infol, conf, init, events...); err != nil {
 				return fmt.Errorf("process: %w", err)
 			}
-
+			h.reportProgress(func() (state terminal.ProgressState, progress float64) {
+				return terminal.ProgressNormal, 0.15
+			})
 			if err := h.assemble(ctx, infol, conf); err != nil {
 				return fmt.Errorf("assemble: %w", err)
 			}
+			h.reportProgress(func() (state terminal.ProgressState, progress float64) {
+				return terminal.ProgressNormal, 0.20
+			})
 
 			return nil
 		}
@@ -830,6 +853,7 @@ func (h *HugoSites) processPartialFileEvents(ctx context.Context, l logg.LevelLo
 
 	changedPaths := struct {
 		changedFiles []*paths.Path
+		addedFiles   []*paths.Path
 		changedDirs  []*paths.Path
 		deleted      []*paths.Path
 	}{}
@@ -896,13 +920,15 @@ func (h *HugoSites) processPartialFileEvents(ctx context.Context, l logg.LevelLo
 			changedPaths.deleted = append(changedPaths.deleted, pss...)
 		} else if ev.isChangedDir {
 			changedPaths.changedDirs = append(changedPaths.changedDirs, pss...)
+		} else if ev.added {
+			changedPaths.addedFiles = append(changedPaths.addedFiles, pss...)
 		} else {
 			changedPaths.changedFiles = append(changedPaths.changedFiles, pss...)
 		}
 	}
 
 	// Find the most specific identity possible.
-	handleChange := func(pathInfo *paths.Path, delete, isDir bool) {
+	handleChange := func(pathInfo *paths.Path, delete, add, isDir bool) {
 		switch pathInfo.Component() {
 		case files.ComponentFolderContent:
 			logger.Println("Source changed", pathInfo.Path())
@@ -921,12 +947,14 @@ func (h *HugoSites) processPartialFileEvents(ctx context.Context, l logg.LevelLo
 						if err == nil {
 							f.Close()
 						}
+
 						if err != nil {
 							// Remove all pages and resources below.
-							prefix := pathInfo.Base() + "/"
+							prefix := paths.AddTrailingSlash(pathInfo.Base())
+
 							h.pageTrees.treePages.DeletePrefixAll(prefix)
 							h.pageTrees.resourceTrees.DeletePrefixAll(prefix)
-							changes = append(changes, identity.NewGlobIdentity(prefix+"*"))
+							changes = append(changes, identity.NewGlobIdentity(prefix+"**"))
 						}
 						return err != nil
 					})
@@ -961,7 +989,10 @@ func (h *HugoSites) processPartialFileEvents(ctx context.Context, l logg.LevelLo
 				}
 			}
 
-			addedOrChangedContent = append(addedOrChangedContent, pathChange{p: pathInfo, structural: delete, isDir: isDir})
+			structural := delete
+			structural = structural || (add && pathInfo.IsLeafBundle())
+
+			addedOrChangedContent = append(addedOrChangedContent, pathChange{p: pathInfo, structural: structural, isDir: isDir})
 
 		case files.ComponentFolderLayouts:
 			tmplChanged = true
@@ -1017,6 +1048,7 @@ func (h *HugoSites) processPartialFileEvents(ctx context.Context, l logg.LevelLo
 	}
 
 	changedPaths.deleted = removeDuplicatePaths(changedPaths.deleted)
+	changedPaths.addedFiles = removeDuplicatePaths(changedPaths.addedFiles)
 	changedPaths.changedFiles = removeDuplicatePaths(changedPaths.changedFiles)
 
 	h.Log.Trace(logg.StringFunc(func() string {
@@ -1027,6 +1059,11 @@ func (h *HugoSites) processPartialFileEvents(ctx context.Context, l logg.LevelLo
 			sb.WriteString("path: " + p.Path())
 			sb.WriteString("\n")
 		}
+		sb.WriteString("Added:\n")
+		for _, p := range changedPaths.addedFiles {
+			sb.WriteString("path: " + p.Path())
+			sb.WriteString("\n")
+		}
 		sb.WriteString("Changed:\n")
 		for _, p := range changedPaths.changedFiles {
 			sb.WriteString("path: " + p.Path())
@@ -1072,15 +1109,19 @@ func (h *HugoSites) processPartialFileEvents(ctx context.Context, l logg.LevelLo
 	}
 
 	for _, deleted := range changedPaths.deleted {
-		handleChange(deleted, true, false)
+		handleChange(deleted, true, false, false)
+	}
+
+	for _, id := range changedPaths.addedFiles {
+		handleChange(id, false, true, false)
 	}
 
 	for _, id := range changedPaths.changedFiles {
-		handleChange(id, false, false)
+		handleChange(id, false, false, false)
 	}
 
 	for _, id := range changedPaths.changedDirs {
-		handleChange(id, false, true)
+		handleChange(id, false, false, true)
 	}
 
 	for _, id := range changes {
